import random
from django.http import HttpResponse
from django.shortcuts import render, redirect
from django.contrib import messages
from django.contrib.auth.models import User, auth
from django.contrib.auth import authenticate, login, logout
from django.contrib.auth import update_session_auth_hash
from django.contrib.auth.forms import PasswordChangeForm
from requests import session
from admin.models import Seller,Product,Top_category,Main_banner,Mid_category,End_category 
from account.models import Customer,Wishlist,Cart,Addresses,Cards,Order,Notification
from django.db.models import F,Sum,Count
from django.contrib.auth.hashers import make_password, check_password
from django.conf import settings
from datetime import datetime,timedelta
import random
import io
from django.http import FileResponse
from reportlab.pdfgen import canvas

todaydate = datetime.today().strftime('%Y-%m-%d')
todaytime = datetime.today().strftime('%H:%M:%S')
# import hashlib

# Create your views here.

def user_index(request):
    if request.session.has_key('sellerId') and request.session.has_key('customerId'):
        seller_id = request.session.get('sellerId')
        tblSeller = Seller.objects.filter(seller_id=seller_id)
        products = Product.objects.filter(sellerId=seller_id)
        topcat_id = request.session.get('topcatId')
        user = Customer.objects.filter(id=request.session.get('customerId'))
        banners = Main_banner.objects.filter(sellerId=seller_id)
        bproducts = Order.objects.filter(seller_id=seller_id).annotate(count=Count('product_id')).order_by('-count').values('product_id').distinct()
        bestproducts = Product.objects.filter(sellerId=seller_id,skuId__in=bproducts).order_by('quantity')
        newproducts = Product.objects.filter(sellerId=seller_id,date__range = [datetime.today(), datetime.today() + timedelta(days=5)]).order_by('date')
        dealsproducts = Product.objects.filter(sellerId=seller_id,category__contains = "Deals")
        return render(request, "user-index.html",{'user': user,'dealsproducts':dealsproducts,'bestproducts':bestproducts,'tblSeller':tblSeller,'newproducts':newproducts,'products':products,'banners':banners,'seller_id':seller_id,'topcat':topcat_id})
    else:
        return redirect('login')

def login(request):
    if request.session.has_key('customerId'):
        return redirect('user-index')
    else:
        request.session['sellerId'] = settings.SELLER_ID
        tblSeller = Seller.objects.filter(seller_id=settings.SELLER_ID)
        return render(request, "login.html",{'tblSeller':tblSeller})   


def slogin(request):
    seller_id = request.session.get('sellerId')
    tblSeller = Seller.objects.filter(seller_id=seller_id)
    return render(request, "seller-login.html",{'tblSeller':tblSeller})

def register(request):
    seller_id = request.session.get('sellerId')
    tblSeller = Seller.objects.filter(seller_id=seller_id)
    return render(request, "register.html",{'tblSeller':tblSeller})


def search(request):
    return render(request, "search.html")


def search_item(request):
    seller_id = request.session.get('sellerId')
    search = request.POST.get('search','')
    products = Product.objects.filter(sellerId=seller_id,productName__icontains=search)
    return render(request, "search-item.html",{'search':search,'products':products,'length':len(products)})
        
def notification(request):
    return render(request, "notification.html")

def notification_add(request,pk):
    myproduct = Product.objects.filter(skuId=pk)
    mynotification = Notification(
                    product_id = pk,
                    customer_id = request.session.get('customerId'),
                    seller_id = request.session.get('sellerId'),
                    product_name = myproduct[0].productName,
                    productImage = myproduct[0].productImage,
                )
    mynotification.save()
    messages.success(request, "We’ll notify you when this product is back in stock")
    return product(request,pk)


def wishlist(request):
    if request.session.get('customerId'):
        wishlist = Wishlist.objects.filter(customer_id=request.session.get('customerId'))
        user = Customer.objects.filter(id=request.session.get('customerId'))
        if user[0].wishlist_quantity == 0:
            return redirect('empty-wishlist')
        return render(request,'wishlist.html',{'wishlist':wishlist,'user':user})
    else:
        return redirect('/')

def wishlist_add(request, pk):
    if request.session.get('customerId'):
        quantity = Customer.objects.filter(id=request.session.get('customerId'))[0].wishlist_quantity
        if quantity <= 25:
            if Wishlist.objects.filter(product_id=pk,customer_id = request.session.get('customerId')).exists() == False:
                product = Product.objects.filter(skuId=pk)
                mywishlist = Wishlist(
                    product_id = product[0].skuId,
                    customer_id = request.session.get('customerId'),
                    seller_id = request.session.get('sellerId'),
                    product_name = product[0].productName,
                    current_price = product[0].currentPrice,
                    old_price = product[0].oldPrice,
                    discount = product[0].discount,
                    productImage = product[0].productImage,
                    status = product[0].status
                )
                mywishlist.save()
                Customer.objects.filter(id=request.session.get('customerId')).update(wishlist_quantity=F('wishlist_quantity')+1)
                return redirect('wishlist')        
            else:
                messages.info(request, "Product is already in Wishlist...")
                return redirect('wishlist')        
        else:
            messages.info(request, "Wishlist is full...")
            return redirect('wishlist')  
    else:
        return redirect('/')      
    

def wishlist_remove(request,pk):
    quantity = Customer.objects.filter(id=request.session.get('customerId'))[0].wishlist_quantity
    if quantity>0:
        Wishlist.objects.filter(product_id=pk,customer_id=request.session.get('customerId')).delete()
        Customer.objects.filter(id=request.session.get('customerId')).update(wishlist_quantity=F('wishlist_quantity')-1)
    return redirect('wishlist')
        

def cart(request):
    if request.session.get('customerId'):
        cart = Cart.objects.filter(customer_id=request.session.get('customerId'))
        user = Customer.objects.filter(id=request.session.get('customerId'))
        if user[0].cart_quantity == 0:
            return redirect('empty-cart')
        total_sum,total_old = 0,0
        for prod in cart:
            total_sum += prod.old_price * prod.quantity 
            total_old += prod.current_price * prod.quantity     
        total_discount = total_sum-total_old
        return render(request,'cart.html',{'cart':cart,'user':user,'total_old':total_old,'total_sum':total_sum,"total_discount":total_discount})
    else:
        return redirect('/')
    
    
def cart_add(request, pk):
    quantity = Customer.objects.filter(id=request.session.get('customerId'))[0].cart_quantity
    if quantity<=25:
        product = Product.objects.filter(skuId=pk)
        sizes = None if product[0].size=="[]" else product[0].size.strip("]'[").split("', '")
        colors = None if product[0].color=="[]" else product[0].color.strip("]'[").split("', '")
        size = sizes[0] if request.method != 'POST' else request.POST['size']
        color = colors[0] if request.method != 'POST' else request.POST['color']
        if Cart.objects.filter(product_id=pk,customer_id = request.session.get('customerId'),size=size,color=color).exists() == False:
            if request.method != 'POST':
                mycart = Cart(
                    product_id = product[0].skuId,
                    customer_id = request.session.get('customerId'),
                    seller_id = request.session.get('sellerId'),
                    product_name = product[0].productName,
                    current_price = product[0].currentPrice,
                    old_price = product[0].oldPrice,
                    discount = product[0].discount,
                    productImage = product[0].productImage,
                    status = product[0].status,
                    size = sizes[0],
                    color = colors[0],
                )
                Customer.objects.filter(id=request.session.get('customerId')).update(cart_quantity=F('cart_quantity')+1)
            else:
                size = request.POST['size']  if request.POST['size'] != "" else sizes[0]  
                color = request.POST['color']  if request.POST['color'] !=""  else colors[0]
                mycart = Cart(
                    product_id = product[0].skuId,
                    customer_id = request.session.get('customerId'),
                    seller_id = request.session.get('sellerId'),
                    product_name = product[0].productName,
                    current_price = product[0].currentPrice,
                    old_price = product[0].oldPrice,
                    discount = product[0].discount,
                    productImage = product[0].productImage,
                    status = product[0].status,
                    size = size,
                    color = color,
                    quantity = request.POST.get('quantity'),
                )
                Customer.objects.filter(id=request.session.get('customerId')).update(cart_quantity=F('cart_quantity')+1)
            mycart.save()
            return redirect('cart')
        else:
            Cart.objects.filter(product_id=pk,customer_id = request.session.get('customerId'),size=size,color=color).update(quantity=F('quantity')+1)
            return redirect('cart')
    else:
        messages.error(request, "Cart is full...")
        return redirect('cart')
    return redirect('cart')
      
def cart_edit(request,pk):
    if request.method == 'POST':
        if 'quantity' not in request.POST:
            Cart.objects.filter(id=pk).update(size=request.POST['size'])
        if 'size' not in request.POST:
            Cart.objects.filter(id=pk).update(quantity=request.POST['quantity'])
    return redirect('cart')
              
            
def cart_to_wishlist(request,pk):
    quantity = Customer.objects.filter(id=request.session.get('customerId'))[0].cart_quantity
    cart = Cart.objects.filter(customer_id=request.session.get('customerId'))
    if quantity>0:
        Cart.objects.filter(product_id=pk,customer_id=request.session.get('customerId')).delete()
        Customer.objects.filter(id=request.session.get('customerId')).update(cart_quantity=F('cart_quantity')-1)
    return wishlist_add(request,pk)
    
    
def cart_remove(request,pk):
    quantity = Customer.objects.filter(id=request.session.get('customerId'))[0].cart_quantity
    if quantity>0:
        Cart.objects.filter(id=pk,customer_id=request.session.get('customerId')).delete()
        Customer.objects.filter(id=request.session.get('customerId')).update(cart_quantity=F('cart_quantity')-1)
    return redirect('cart')


def delivery(request):
    if request.method == "POST":
        mycart = Cart.objects.filter(customer_id=request.session.get('customerId'),seller_id=request.session.get('sellerId'))
        tblAddresses = Addresses.objects.filter(seller_id = request.session.get('sellerId'),customer_id = request.session.get('customerId'))
        bagtotal = request.POST['bagtotal']
        savings = request.POST['savings']
        coupon = request.POST['coupon']
        delivery = request.POST['delivery']
        total = request.POST['total']
        return render(request, "delivery.html",{'mycart':mycart,'tblAddresses':tblAddresses,'bagtotal':bagtotal,'savings':savings,'coupon':coupon,'delivery':delivery,'total':total})
    else:
        return redirect('cart')


def payment(request):
    if request.method == "POST":
        bagtotal = request.POST['bagtotal']
        savings = request.POST['savings']
        coupon = request.POST['coupon']
        delivery = request.POST['delivery']
        total = request.POST['total']
        address = request.POST['address']
        return render(request, "payment.html",{'bagtotal':bagtotal,'savings':savings,'coupon':coupon,'delivery':delivery,'total':total,'address':address})
    else:
        return redirect('cart')


def category(request):
    categories = Mid_category.objects.filter(tcat_id=request.session.get('topcatId'))
    user = Customer.objects.filter(id=request.session.get('customerId'))
    return render(request, "category.html",{'user':user,'categories':categories})


def product(request,pk):
    if request.session.has_key('customerId'):
        seller_id = request.session.get('sellerId')
        product = Product.objects.filter(sellerId=seller_id,skuId=pk)
        imgs = [] if product[0].otherProductImage=="[]" else product[0].otherProductImage.strip("]'[").split("', '")
        colors = [] if product[0].color=="[]" else product[0].color.strip("]'[").split("', '")
        sizes = [] if product[0].size=="[]" else product[0].size.strip("]'[").split("', '")
        user = Customer.objects.filter(id=request.session.get('customerId'))
        return render(request, "product.html",{'product':product,'user':user,'imgs':imgs,'colors':colors,'sizes':sizes})
    else:
        return redirect('/')
        
def profile(request):
    if request.session.has_key('customerId'):
        user = Customer.objects.filter(id=request.session.get('customerId'))
        return render(request, "profile.html",{'user':user})
    else:
        return redirect('/')


def profile_edit(request):
    if request.session.has_key('customerId'):
        user = Customer.objects.filter(id=request.session.get('customerId'))
        return render(request, "profile-edit.html", {'user':user})
    else:
        return redirect('/')
    
    
def order_placed(request):
    if request.method == "POST" and Customer.objects.filter(id=request.session.get('customerId'))[0].cart_quantity != 0:
        mycart = Cart.objects.filter(customer_id=request.session.get('customerId'),seller_id=request.session.get('sellerId'))
        tblAddresses = Addresses.objects.filter(id=request.POST['address'])
        bagtotal = request.POST['bagtotal']
        savings = request.POST['savings']
        coupon = request.POST['coupon']
        delivery = request.POST['delivery']
        total = request.POST['total']
        orderid = "OD"+str(random.randrange(10000000, 99999999))
        for cart in mycart:
            myorder = Order(
                order_id = orderid,
                product_id = cart.product_id,
                customer_id = cart.customer_id,
                seller_id = cart.seller_id,
                product_name = cart.product_name,
                current_price = cart.current_price,
                old_price = cart.old_price,
                discount = cart.discount,
                productImage = cart.productImage,
                size = cart.size,
                color = cart.color,
                quantity = cart.quantity,
                ordered = todaydate,
                delivery_status = "Ordered",
                delivery = request.POST['address'],
                bag_total = bagtotal,
                delivery_charges = delivery,
                bag_savings = savings,
                total = total[1:],
            )
            myorder.save()
        id = Order.objects.filter(order_id=orderid)[0].id
        Cart.objects.filter(customer_id=request.session.get('customerId'),seller_id=request.session.get('sellerId')).delete()
        Customer.objects.filter(id=request.session.get('customerId')).update(cart_quantity=0)
        return render(request, "order-placed.html",{'id':id,'orderid':orderid,'mycart':mycart,'tblAddresses':tblAddresses,'bagtotal':bagtotal,'savings':savings,'coupon':coupon,'delivery':delivery,'total':total})
    else:
        return redirect('cart')
        

def order_tracking(request,pk):
    order = Order.objects.filter(id=pk)
    otherOrders = Order.objects.filter(order_id=order[0].order_id).exclude(id=pk)
    address = Addresses.objects.filter(id=order[0].delivery)
    print(address)
    return render(request, "order-tracking.html",{'order':order,'address':address,'otherOrders':otherOrders})


def order_history(request):
    seller_id = request.session.get('sellerId')
    orders = Order.objects.filter(seller_id=seller_id)
    if len(orders)==0:
        return redirect('no-orders')
    return render(request, "order-history.html",{'orders':orders})


def forgot_password(request):
    seller_id = request.session.get('sellerId')
    tblSeller = Seller.objects.filter(seller_id=seller_id)
    return render(request, "forgot-password.html", {'tblSeller':tblSeller})


def shop(request,val):
    seller_id = request.session.get('sellerId')
    user = Customer.objects.filter(id=request.session.get('customerId'))
    if val == "U499":
        products = Product.objects.filter(sellerId=seller_id,currentPrice__lte = 499)
    elif val == "200OFF":
        products = Product.objects.filter(sellerId=seller_id).annotate(off = F('oldPrice') - F('currentPrice')).filter(off__gt = 200)
    elif val == "50":
        products = Product.objects.filter(sellerId=seller_id,discount__lte = 50)
    elif val == "25":
        products = Product.objects.filter(sellerId=seller_id,discount__lte = 25)
    elif val == "30":
        products = Product.objects.filter(sellerId=seller_id,discount__lte = 30)
    elif val == "Buy":
        products = Product.objects.filter(sellerId=seller_id,category__contains = val)
    elif val == "Deals":
        products = Product.objects.filter(sellerId=seller_id,category__contains = val)
    else:
        products = Product.objects.filter(sellerId=seller_id)
    return render(request, "shop.html",{'products':products,'user':user})


def saved_address(request):
    tblAddresses = Addresses.objects.filter(seller_id = request.session.get('sellerId'),customer_id = request.session.get('customerId'))
    return render(request, "saved-address.html", {'tblAddresses':tblAddresses})

def add_address(request):
    if request.method == 'POST' and request.session.get('customerId'):
        addr = Addresses(
            customer_id = request.session.get('customerId'),
            seller_id = request.session.get('sellerId'),
            name = request.POST['name'],
            phone = request.POST['phone'],
            pincode = request.POST['pin'],
            house = request.POST['house'],
            area = request.POST['area'],
            landmark = request.POST['landmark'],
            town = request.POST['town'],
            state = request.POST['state'],
            typeof = request.POST['type'],
            default = request.POST['default'],
        )
        addr.save()
        return redirect('saved-address')
    else:
        return redirect('/')

def edit_address(request,pk):
    if request.method == 'POST' and request.session.get('customerId'):
        Addresses.objects.filter(id=pk).update(
            customer_id = request.session.get('customerId'),
            seller_id = request.session.get('sellerId'),
            name = request.POST['name'],
            phone = request.POST['phone'],
            pincode = request.POST['pin'],
            house = request.POST['house'],
            area = request.POST['area'],
            landmark = request.POST['landmark'],
            town = request.POST['town'],
            state = request.POST['state'],
            typeof = request.POST['type'],
            default = request.POST['default'],
        )
        return redirect('saved-address')
    tblAddresses = Addresses.objects.filter(id=pk)
    return render(request, "edit-address.html", {'tblAddresses':tblAddresses})
    
def remove_address(request,pk):
    Addresses.objects.filter(id=pk).delete()
    return redirect('saved-address')
    
    
def new_address(request):
    return render(request, "new-address.html")


def reset_password(request):
    return render(request, "reset-password.html")

def terms_condition(request):
    return render(request, "terms-condition.html")
    
def help(request):
    return render(request, "help.html")

def setting(request):
    return render(request, "settings.html")

def about_us(request):
    return render(request, "about-us.html")
    
def pages(request):
    return render(request, "pages.html")

def accordion(request):
    return render(request, "accordion.html")

def alert(request):
    return render(request, "alert.html")

def buttons(request):
    return render(request, "buttons.html")

def form_style(request):
    return render(request, "form-style.html")

def modals_offcanvas(request):
    return render(request, "modals-offcanvas.html")

def pricing_slider(request):
    return render(request, "pricing-slider.html")

def tabs(request):
    return render(request, "tabs.html")

def inner_category(request,pk):
    categories = End_category.objects.filter(mcat_id=pk)
    user = Customer.objects.filter(id=request.session.get('customerId'))
    return render(request, "inner-category.html",{'user':user,'categories':categories})

def onboarding(request):
    return render(request, "onboarding.html")

def empty_cart(request):
    user = Customer.objects.filter(id=request.session.get('customerId'))
    return render(request, "empty-cart.html",{'user':user})

def empty_wishlist(request):
    user = Customer.objects.filter(id=request.session.get('customerId'))
    return render(request, "empty-wishlist.html",{'user':user})

def coupons(request):
    return render(request, "coupons.html")

def no_orders(request):
    return render(request, "no-orders.html")

def saved_cards(request):
    tblCards = Cards.objects.filter(seller_id = request.session.get('sellerId'),customer_id = request.session.get('customerId'))
    return render(request, "saved-cards.html", {'tblCards':tblCards})

def edit_card(request,pk):
    if request.method == 'POST' and request.session.get('customerId'):
        Cards.objects.filter(id=pk).update(
            customer_id = request.session.get('customerId'),
            seller_id = request.session.get('sellerId'),
            card_type = request.POST['cardtype'],
            bank = request.POST['bank'],
            number = request.POST['number'],
            name = request.POST['name'],
            valid = request.POST['valid']
        )
        return redirect('saved-cards')
    tblCards = Cards.objects.filter(id=pk)
    return render(request, "edit-card.html", {'tblCards':tblCards})
    
def remove_card(request,pk):
    Cards.objects.filter(id=pk).delete()
    return redirect('saved-cards')

def invoice(request,pk):
    buffer = io.BytesIO()

    p = canvas.Canvas(buffer)
    invoice = str(random.randrange(1000000000000, 9999999999999))
    order = Order.objects.filter(id=pk)
    p.drawString(270, 800, "INVOICE")
    p.drawString(100, 700, "Invoice Number : IN"+invoice)
    p.drawString(100, 685, "Order Number : "+str(order[0].order_id))
    p.drawString(100, 670, "Total Bill : "+str(order[0].total))

    p.showPage()
    p.save()

    buffer.seek(0)
    return FileResponse(buffer, as_attachment=True, filename='IN'+invoice+'.pdf')


def user_register(request):
    if request.method == 'POST' and request.session.has_key('sellerId'):
        username = request.POST['name']
        email = request.POST['email']
        contact = request.POST['contact']
        password = request.POST['password']
        seller_id = request.session.get('sellerId')
        if(email==""):
            email=None
        encryptedpassword = make_password(password)
        if username == username:
            if Customer.objects.filter(email=email,seller_id=settings.SELLER_ID).exists():
                messages.error(request, "Email Already Exists")
                return redirect('register/')
            elif Customer.objects.filter(contact=contact,seller_id=settings.SELLER_ID).exists():
                messages.error(request, "Contact Already Exists")
                return redirect('register/')
            else:
                user = Customer(
                    password=encryptedpassword, 
                    username=username, 
                    contact=contact,
                    email=email, 
                    is_active=1,
                    date_joined=todaydate,
                    seller_id=seller_id)
                user.save()
                messages.success(request, 'Account created successfully')
                return redirect('/')
        else:
            messages.error(request, 'User name')
            return redirect('register')
    else:
        return render(request, 'register')
    

def user_login(request):
    if request.method == 'POST':
        if request.session.has_key('sellerId'):
            username = request.POST['username']
            upassword = request.POST['password']
            if('@' in username):
                query_set = Customer.objects.filter(email=username,seller_id=settings.SELLER_ID)
            elif(username.isnumeric()):
                query_set = Customer.objects.filter(contact=username,seller_id=settings.SELLER_ID)
            
            if(len(query_set)!=0):
                customerId = query_set[0].id
                username = query_set[0].contact
                seller_id = request.session.get('sellerId')
                tblSeller = Seller.objects.filter(seller_id=seller_id)
                topcat_id = tblSeller[0].top_cat_id
                request.session['topcatId'] = topcat_id
                products = Product.objects.all()
                password = query_set[0].password
                decryptedpassword = check_password(upassword, password)
            else:
                username = None
            if  username is not None and decryptedpassword == True:
                request.session['customerId'] = customerId
                return user_index(request)
            else:
                messages.info(request,"Username and password invalid! Please try again")
                return redirect('/')
        else:
            return redirect('/')
    else:
        return redirect('/')


def logout(request):
    try:
        del request.session['customerId']
    except:
        pass
    return redirect('login')


def update_user(request):
    if request.method == 'POST' and request.session.has_key('customerId'):
        Customer.objects.filter(id=request.session.get('customerId')).update(
            username = request.POST['name'],
            email = request.POST['email'],
            contact = request.POST['number'],
            dob = request.POST['date'],
            gender = request.POST['gender'],
        )
        # to update the image need to apply save command to sabe image into location
        user = Customer.objects.get(id=request.session.get('customerId'))
        user.userImg = request.FILES.get('myimage')
        user.save()
        messages.success(request, 'Data updated successfully')
        return redirect('profile-edit')
    else:
        messages.error(request, 'Something went wrong')
        return redirect('profile-edit')


def changepassword(request):
    if request.method == "POST" and request.session.has_key('customerId'):
        upassword = request.POST['oldPassward']
        password = Customer.objects.filter(id=request.session.get('customerId'))[0].password
        decryptedpassword = check_password(upassword, password)
        if decryptedpassword:
            if request.POST['newPassward'] == request.POST['confnewPassward']:
                newPassword =  request.POST['newPassward']
                encryptedpassword = make_password(newPassword)
                Customer.objects.filter(id=request.session.get('customerId')).update(
                    password=encryptedpassword,
                )
                messages.success(request, 'Your password was successfully changed..')
            else:
                messages.error(request, "New password and confirm password didn't matched...")
        else:
            messages.error(request, 'Your current password was invalid..')
    return redirect("reset-password")


def seller_login(request):
    if request.method == 'POST':
        uemail = request.POST['email']
        upassword = request.POST['password']
        seller_id = request.session.get('sellerId')
        tblSeller = Seller.objects.filter(seller_id=seller_id)        
        products = Product.objects.all()

        logemail, password = "", ""
        for log in tblSeller:
            logemail = log.email
            password = log.password
            logstatus = log.status
            gain_seller_id = log.seller_id
            topcat_id = log.top_cat_id
            
        decryptedpassword = check_password(upassword, password)
        
        if logemail == uemail and decryptedpassword == True and gain_seller_id == seller_id:
            if logstatus == 1:
                return render(request, 'seller-index.html',{'tblSeller':tblSeller,'products':products,'seller_id':seller_id,'topcat':topcat_id})
            else:
                messages.info(request, "You are not active seller!")
                return redirect('/seller_login/')
        else:
            messages.info(
                request, "Username and password invalid! Please try again")
            return redirect('/seller_login')
    else:
        return redirect('/seller_login')
